/*
 * (c) Copyright 1992 by Panagiotis Tsirigotis
 * All rights reserved.  The file named COPYRIGHT specifies the terms 
 * and conditions for redistribution.
 */

#ifndef CONFIG_H
#define CONFIG_H

/*
 * $Id: config.h,v 5.1 1992/10/31 23:59:07 panos Exp $
 */

#define DEFAULT_SYSLOG_FACILITY  	LOG_DAEMON

#define DEFAULT_CONFIG_FILE			"/etc/xinetd.conf"
#define DUMP_FILE							"/tmp/xinetd.dump"

/*
 * Max number of concurrently running processes forked to get 
 * the user id (RFC 931) from the remote host
 */
#define DEFAULT_LOGPROCS				15

/*
 * Max number of concurrently running processes forked to do service shutdown
 */
#define DEFAULT_SHUTDOWNPROCS			10

/*
 * This is used when an instance limit is not specified for a service
 * and the defaults entry does not specify an instance limit either.
 * It can be a positive number or "UNLIMITED".
 */
#define DEFAULT_INSTANCE_LIMIT		UNLIMITED

/*
 * DEFAULT_LOOP_RATE is a bound on the rate of forking servers for a
 * particular service. If that rate is exceeded, the service is deactivated.
 * We check this rate on time intervals of duration LOOP_INTERVAL seconds.
 */
#define DEFAULT_LOOP_RATE				10
#define LOOP_INTERVAL					2

/*
 * DEFAULT_SERVICE_SYSLOG_LEVEL is the level used for log messages when a
 * service logs to syslog
 */
#define DEFAULT_SERVICE_SYSLOG_LEVEL	LOG_INFO

/*
 * LOG_OPEN_FLAGS are the flags used to open a log file
 */
#define LOG_OPEN_FLAGS					O_CREAT+O_APPEND+O_WRONLY

/*
 * Number of consecutive fork failures that we are willing to put up with
 * before giving up.
 */
#define MAX_FORK_FAILURES        	5

#define DUMP_FILE_MODE					0644
#define LOG_FILE_MODE         		0644

/*
 * Signal-to-action mapping
 */
#define RECONFIG_SOFT_SIG				SIGUSR1
#define RECONFIG_HARD_SIG				SIGUSR2
#define TERMINATION_SIG					SIGTERM
#define STATE_DUMP_SIG					SIGHUP
#define CONSISTENCY_CHECK_SIG			SIGIOT
#define RETRY_SIG							SIGALRM
#define SERVER_EXIT_SIG					SIGCHLD
#define QUIT_SIG							SIGQUIT

/*
 * The DATAGRAM_SIZE is big enough for an ethernet packet
 */
#define DATAGRAM_SIZE					2048

/*
 * Time interval between retry attempts
 */
#define RETRY_INTERVAL					1		/* second */

/*
 * LOG_EXTRA_MIN, LOG_EXTRA_MAX define the limits by which the hard limit
 * on the log size can exceed the soft limit
 */
#define LOG_EXTRA_MIN					(  5 * 1024 )
#define LOG_EXTRA_MAX					( 20 * 1024 )

#endif	/* CONFIG_H */
